/**
 * OW2 Util
 * Copyright (C) 2008 SERLI
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id Maven2ResourceFetcher.java $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.plan.fetcher.impl.maven2;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;

import org.ow2.util.file.FileUtils;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.plan.bindings.deploymentplan.DeploymentHelper;
import org.ow2.util.plan.bindings.deploymentplan.maven2.Maven2Deployment;
import org.ow2.util.plan.bindings.exceptions.InvalidDeploymentException;
import org.ow2.util.plan.bindings.repository.Repository;
import org.ow2.util.plan.bindings.repository.RepositoryKind;
import org.ow2.util.plan.fetcher.api.IResourceFetcher;
import org.ow2.util.plan.fetcher.api.exceptions.FetcherException;
import org.ow2.util.plan.fetcher.api.exceptions.ResourceFetcherNotResolvedException;
import org.ow2.util.plan.fetcher.api.exceptions.ResourceNotFoundException;
import org.ow2.util.plan.fetcher.impl.AbsResourceFetcherImpl;

/**
 * @author Mickaël LEDUQUE
 */
public class Maven2ResourceFetcher extends AbsResourceFetcherImpl implements IResourceFetcher {

    /**
     * The logger.
     */
    private static final Log logger = LogFactory.getLog(Maven2ResourceFetcher.class);

    /**
     * The local copy of the resource.
     */
    private File localFile = null;

    /**
     * The URL of the resource.
     */
    private URL foundURL = null;

    /**
     * {@inheritDoc}
     */
    public RepositoryKind getSupportedRepositoryType() {
        return RepositoryKind.MAVEN_2;
    }

    /**
     * {@inheritDoc}
     */
    public File getResource() throws ResourceFetcherNotResolvedException {
        if (this.localFile == null) {
            throw new ResourceFetcherNotResolvedException();
        }
        return this.localFile;
    }

    /**
     * {@inheritDoc}
     */
    public boolean hasChanged() throws FetcherException {
        try {
            URLConnection connection = foundURL.openConnection();
            if (connection.getLastModified() > localFile.lastModified()) {
                return true;
            }
        } catch (IOException e) {
            logger.debug("Cannot retrieve the last modification date for the URL {0} : {1}", foundURL, e.getMessage());
        }

        return false;
    }

    /**
     * {@inheritDoc}
     */
    public void resolve() throws FetcherException {
        if (this.deployment == null) {
            throw new FetcherException(new InvalidDeploymentException("Null deployment"));
        }
        if (!(this.deployment instanceof Maven2Deployment)) {
            throw new FetcherException(new InvalidDeploymentException("Bad deployment type: "
                    + this.deployment.getClass().getName()));
        }
        Maven2Deployment maven2Deployment = (Maven2Deployment) this.deployment;

        logger.debug("Trying to resolve resource {0}", maven2Deployment.toString());

        // Get artifact properties
        String groupId = maven2Deployment.getGroupId();
        String artifactId = maven2Deployment.getArtifactId();
        String version = maven2Deployment.getVersion();
        String type = maven2Deployment.getType();
        String classifier = maven2Deployment.getClassifier();

        if (groupId == null || artifactId == null || version == null) {
            throw new FetcherException(new InvalidDeploymentException("Incomplete maven description: " + maven2Deployment));
        }

        String repositoryRef = null;
        try {
            repositoryRef = DeploymentHelper.getRepositoryRef(maven2Deployment);
        } catch (InvalidDeploymentException e) {
            logger.error(e.getMessage());
        }

        // Default artifact type is jar.
        if (type == null) {
            type = "jar";
        }

        if (repositoryRef != null) {
            resolveArtifact(getRepositoryManager().getRepositoryById(repositoryRef), groupId, artifactId, version, classifier,
                    type);
        } else {
            // Parse the (maven) repository list
            for (Repository repository : this.getRepositoryManager()) {
                if (repository.getType() == RepositoryKind.MAVEN_2) {
                    resolveArtifact(repository, groupId, artifactId, version, classifier, type);
                    if (localFile != null && localFile.exists()) {
                        // the resource was found, we can put an end to the
                        // search.
                        return;
                    }
                }
            }
        }

        if (localFile == null || !localFile.exists()) {
            throw new FetcherException(new ResourceNotFoundException(maven2Deployment.toString()));
        }
    }

    /**
     * Try to resolve the requested artifact on the given repository.
     * @param repository Repository path
     * @param groupId Artifact groupId
     * @param artifactId Artifact ID
     * @param version Artifact version
     * @param classifier Artifact's classifier (may be null)
     * @param type Artifact type
     */
    private void resolveArtifact(final Repository repository, final String groupId, final String artifactId,
            final String version, final String classifier, final String type) throws FetcherException {
        URL repositoryURL = null;
        try {
            repositoryURL = new URL(repository.getUrl());
        } catch (MalformedURLException e) {
            logger.debug("Invalid repository URL {0}", repository);
        }

        // If the repository is local, try to return the local file
        if (repositoryURL != null && "file".equals(repositoryURL.getProtocol())) {
            localFile = Maven2Utils.getMaven2File(repositoryURL.getFile(), groupId, artifactId, version, classifier, type);
        } else {
            try {
                foundURL = Maven2Utils.getMaven2URL(repositoryURL.toExternalForm(), groupId, artifactId, version, classifier,
                        type);
            } catch (MalformedURLException e) {
                logger.debug("Invalid resource URL:" + e.getMessage());
            }

            // Create a local copy of the resource.
            localFile = Maven2Utils.getMaven2File(this.localRepositoriesBaseDir + File.separator + repository.getId(), groupId,
                    artifactId, version, classifier, type);

            // Create the local file path.
            if (!localFile.exists() || hasChanged()) {
                if (!localFile.getParentFile().exists()) {
                    localFile.getParentFile().mkdirs();
                }

                InputStream is = null;

                try {
                    URLConnection connection = foundURL.openConnection();
                    is = connection.getInputStream();
                    // Create the local file
                    FileUtils.dump(is, localFile);

                    // The local file has the same last modified date than the
                    // remote resource.
                    // This is useful to check if the remote resource has been
                    // modified since the last copy.
                    localFile.setLastModified(connection.getLastModified());
                } catch (Exception e) {
                    logger.debug("Exception while copying data from resource {0} : {1}", foundURL, e.getMessage());
                } finally {
                    if (is != null) {
                        try {
                            // Close input stream
                            is.close();
                        } catch (IOException e) {
                            // Nothing to do
                        }
                    }
                }
            }
        }
    }

    /**
     * {@inheritDoc}
     */
    public void update() throws FetcherException {
        this.resolve();
    }
}
